import { useState, useRef, useEffect, useMemo } from "react";
import { LogStreamConsole } from "../common/LogStream/LogStreamConsole";
import { LogStreamConsoleDialog } from "../common/LogStream/LogStreamConsoleDialog";
import { PopoutButton } from "../common/Buttons/PopoutButton";
import { Alert, LinearProgress, Stack } from "@mui/material";
import { getErrorMessage, isAbortError } from "../common/Helper";
import { useLazyFetchDeploymentJobLogQuery } from "../../store/deployApi";

const LOG_POLL_INTERVAL = 5000;

export function DeploymentJobOutput({ job, isExpanded }) {
  const [consoleDialogOpen, setConsoleDialogOpen] = useState(false);
  const finalResultsNeeded = useRef(true);
  const params = useMemo(
    () => ({
      awxJobId: job.awxJobId
    }),
    [job]
  );
  const [getLogById, { data: log, isSuccess: logDataReady, error: logError }] =
    useLazyFetchDeploymentJobLogQuery({
      pollingInterval: isExpanded ? LOG_POLL_INTERVAL : 0
    });

  useEffect(() => {
    if (isExpanded) {
      if (!job.finishedAt || finalResultsNeeded.current) {
        getLogById(params);
        finalResultsNeeded.current = !job.finishedAt;
      }
    }
  }, [job.finishedAt, job.jobId, isExpanded, getLogById, params]);

  const hasAbortError = isAbortError(logError);
  const showLoading = !log || !job.startTime;

  useEffect(() => {
    finalResultsNeeded.current = true;
  }, [job.jobId]);

  if (logError && !hasAbortError) {
    return (
      <Stack>
        <Alert severity="error">{getErrorMessage(logError)}</Alert>
      </Stack>
    );
  }

  if (showLoading) {
    return (
      <Stack>
        <LinearProgress color="primary" />
      </Stack>
    );
  }

  return (
    <Stack>
      <Stack
        display="flex"
        flexDirection="row"
        justifyContent="flex-end"
        sx={{ marginBottom: (theme) => theme.spacing(2) }}
      >
        <PopoutButton
          title="Popout log"
          onButtonClick={() => setConsoleDialogOpen(true)}
        />
      </Stack>
      <LogStreamConsoleDialog
        title="Job log stream"
        loading={showLoading}
        open={consoleDialogOpen}
        log={log?.stdoutHtml}
        dataReady={logDataReady}
        onDialogClose={() => setConsoleDialogOpen(false)}
      />
      <LogStreamConsole
        log={log.stdoutHtml}
        dataReady={logDataReady}
        height="500px"
      />
    </Stack>
  );
}
